from my_package_profile_manager.libraries.user.manager import UserManager
from my_package_profile_manager.libraries.user.validations.user import UserValidationRules
from libraries.system.logger import Log


class EmailVerification():
    def post(self):
        response: dict = {'success_status': False }

        user_data: dict = dict()
        user_data['token'] = self.get_argument('token', None)
        user_data['email'] = self.get_argument('email', None)

        try:
            validation_rules = UserValidationRules()
            errors = validation_rules.email_validation(user_data)
            if len(errors):
                response['status'] = 400
                raise ValueError(errors)

            user_manager = UserManager()
            response = user_manager.account_verification(**user_data)
            if response["success_status"]:
                self.set_data(response['detail'])
                self.set_response_status(self.RESPONSE_STATUS_SUCCESS_TRUE)
            else:
                self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)
            self.set_message(response['message'])

        except ValueError as e:
            self.set_message(str(e))
            self.set_status_code(400)
            self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)

        except Exception as e:
            Log().entry(e)  # loggin error
            self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)

        finally:
            self.set_status_code(response['status'])
            self.send()


class ForgotPassword():
    def post(self):
        response: dict = {'success_status': False }
        user_data: dict = dict()
        user_data["email"] = self.get_argument('email', None)
        if user_data["email"] is not None:
            user_data["email"] = user_data["email"].lower()
        try:
            validation_rules = UserValidationRules()
            errors = validation_rules.email_validation(user_data)
            if len(errors):
                raise ValueError(errors)

            user_manager = UserManager()
            response = user_manager.forget_password(user_data["email"])
            self.set_message(response['message'])

        except ValueError as e:
            self.set_message(str(e))
            self.set_status_code(400)
            self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)

        except Exception as e:
            Log().entry(e)  # loggin error
            self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)

        finally:
            if response["success_status"]:
                self.set_data(response)
                self.set_response_status(self.RESPONSE_STATUS_SUCCESS_TRUE)
            else:
                self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)
            self.set_status_code(response['status'])
            self.send()

class Register():
    def post(self):
        response: dict = {'success_status': False}
        user_data: dict = dict()
        user_data["name"] = self.get_argument('name', None)
        user_data["email"] = self.get_argument('email', None)
        user_data["agree_to_terms"] = self.get_argument('agree_to_terms', None)
        user_data["password"] = self.get_argument('password', None)
        user_data["confirm_password"] = self.get_argument('confirm_password', None)
        try:
            validation_rules = UserValidationRules()
            if user_data['password'] is None:
                user_data['password'] = ''
            if user_data['confirm_password'] is None:
                user_data['confirm_password'] = ''
            errors = validation_rules.register_validation(user_data, user_data['password'], user_data['confirm_password'])
            if len(errors):
                response['status'] = 400
                raise ValueError(errors)

            user_manager = UserManager()
            response = user_manager.user_registration(**user_data)
            self.set_message(response['message'])

        except ValueError as e:
            self.set_message(str(e))
            self.set_status_code(400)
            self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)

        except Exception as e:
            Log().entry(e)  # loggin error
            self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)

        finally:
            if response["success_status"]:
                self.set_data(response)
                self.set_response_status(self.RESPONSE_STATUS_SUCCESS_TRUE)
            else:
                self.set_response_status(self.RESPONSE_STATUS_SUCCESS_FALSE)
            self.set_status_code(response['status'])
            self.send()