import datetime
from libraries.system.model import BaseModel
from my_package_profile_manager.libraries.user.model import UserModel
from peewee import ForeignKeyField, DateTimeField, BooleanField, CharField, TextField, IntegerField
from playhouse.postgres_ext import DateTimeTZField


class ApiKeyModel(BaseModel):
    class Meta:
        table_name = "api_keys"
    app_name = CharField(unique=True, max_length=250, index=True)
    app_description = TextField(null=False)
    key = CharField(unique=True, max_length=250, index=True)
    secret = CharField(unique=True, max_length=250)
    expiry_date = DateTimeTZField(null=True, index=True)
    active = BooleanField(default=False)
    created_at = DateTimeTZField(default=datetime.datetime.now)
    updated_at = DateTimeTZField(default=datetime.datetime.now)


class ApiTokenModel(BaseModel):
    class Meta:
        table_name = "api_tokens"
    user_id = ForeignKeyField(UserModel, null=True, index=True, on_delete='CASCADE')
    api_key_id = ForeignKeyField(ApiKeyModel, index=True, on_delete='CASCADE')
    token = TextField(index=True)
    revoked = BooleanField(default=False)
    created_at = DateTimeTZField(default=datetime.datetime.now)
    updated_at = DateTimeTZField(default=datetime.datetime.now)
