import re
from spotlight.validator import Validator
from my_package_profile_manager.libraries.user.model import UserModel
from libraries.system.logger import Log
from spotlight.rules import Rule

class UniqueEmail(Rule):
    name = "unique_email"

    def passes(self, field: str, value: any, parameters, validator) -> bool:
        self.message_fields = dict(field=field)
        is_unique = True

        try:
            result = UserModel.get_or_none(UserModel.email == value)
            if result is not None:
                is_unique = False
        except Exception as e:
            Log().entry(e)
        finally:
            return is_unique

    @property
    def message(self) -> str:
        return "Email already exists."


class ComparePasswords(Rule):
    """Compare password and confirm password"""
    name = "is_matched"

    def passes(self, field: str, value: any, parameters, validator) -> bool:
        self.message_fields = dict(field=field)
        return parameters[0].__eq__(value)

    @property
    def message(self) -> str:
        return "Password and confirm password did not matched."

class IsValidPassword(Rule):
    name="is_valid_password"

    def passes(self, field: str, value: any, parameters, validator) -> bool:
        self.message_fields = dict(field=field)

        if len(re.findall("[A-Za-z0-9@#$]", parameters[0])) == len(parameters[0]):
            return True
        else:
            return False

    @property
    def message(self) -> str:
        return "{field} is not valid."

class IsChecked(Rule):
    name = "is_checked"

    def passes(self, field: str, value: any, parameters, validator) -> bool:
        self.message_fields = dict(field=field)
        if value == "false":
            return False
        else:
            return True

    @property
    def message(self) -> str:
        return "Please check {field}."


class UserValidationRules():
    def __init__(self):
        self.validator = Validator()

    def register_validation(self, data: dict, password: str,confirm_password: str) -> list:

        self.validator.register_rule(IsValidPassword())
        self.validator.register_rule(UniqueEmail())
        self.validator.register_rule(ComparePasswords())
        self.validator.register_rule(IsChecked())
        rules = {
            "name": "required|string|max:250",
            "email": "required|email|unique_email|max:250",
            "password": "required|string|min:6|max:250|is_valid_password:"+password,
            "confirm_password": "required|string|min:6|max:250|is_valid_password:"+confirm_password+"|is_matched:"+password,
            "agree_to_terms": "required|is_checked"
        }
        """
             if want to override error message
             add message below
             e.g  name.required:"Name is required."
        """
        self.validator.overwrite_messages = {
            "password.is_valid_password":"Password is not valid.",
            "agree_to_terms": "Check I agree to the Terms of use.",
            "confirm_password.is_valid_password": "Confirm password is not valid."
        }
        return self.validator.validate(data, rules)

    def email_validation(self, email: str):
        rules = {
            "email": "required|email|max:250",
        }
        return self.validator.validate(email, rules)

    def token_validation(self, token: str) -> list:
        rules = {
            "token": "required|string|max:250"
        }
        return self.validator.validate(token, rules)