import glob, os, datetime, secrets
from imp import load_source
from application.settings import application_settings
from libraries.system.migration.model import MigrationModel
from libraries.system.migration.manager import MigrationManager
from  my_package_profile_manager.libraries.api.model import ApiKeyModel


class ProfilLibMigrationManager(MigrationManager):

    def _get_defined_migrations(self):
        """ Returns list of all the migrations that are defined in the application """
        current_file = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        migration_dir = current_file+'/migrations'

        defined_migration_list = glob.glob(migration_dir + '/[!_]*.*')
        # print(defined_migration_list)
        defined_migration_dict = {}
        for defined_migration in defined_migration_list:
            migration_name = os.path.basename(defined_migration)
            migration_name = migration_name.replace(".py", "")
            defined_migration_dict[migration_name] = defined_migration

        return defined_migration_dict

    def run(self):
        MigrationModel.create_table(True)
        self._set_pending_migrations()
        self._migrate()

    def _migrate(self):
        total_pending_migrations = len(self._pending_migrations.keys())
        if total_pending_migrations > 0:
            print("%s new migration(s) collected." % total_pending_migrations)
            for migration_name, migration in self._pending_migrations.items():
                try:
                    migration_module = load_source('Migration', migration)
                    migration_module = migration_module.Migration()
                    getattr(migration_module, self.get_migration_action())()
                    print(
                        "migration script %s:%s executed successfully." % (migration_name, self.get_migration_action()))
                except Exception as e:
                    print(e)
                else:
                    if self.get_migration_action() == 'up':
                        MigrationModel.create(name=migration_name, action=self.get_migration_action())
                    if self.get_migration_action() == 'down':
                        MigrationModel.delete().where(MigrationModel.name == migration_name,
                                                      MigrationModel.action == 'up').execute()

            self._save_api_keys()
        else:
            print("No new migration collected.")

    def _save_api_keys(self):
        print("\nStarting to generate api key and secret...\n")
        try:
            key = secrets.token_hex(8)
            secret = secrets.token_hex(18)
            app_name = application_settings['application_name'] + ' ' + application_settings["default_module"].capitalize()
            if key is not None and secret is not None:
                name = (list(application_settings['services'].keys())[0])
                apikey = application_settings["services"][name]["api_key"]
                apisecret = application_settings["services"][name]["api_token"]
                result = ApiKeyModel.get_or_none(ApiKeyModel.key == apikey, ApiKeyModel.secret == apisecret)
                if result is None:
                    api_keys: dict = {}
                    api_keys['app_name'] = app_name
                    api_keys['app_description'] = app_name
                    api_keys['key'] = key
                    api_keys["secret"] = secret
                    api_keys["active"] = True
                    api_keys["updated_at"]= datetime.datetime.now()
                    ApiKeyModel.create(**api_keys)
                    print("Please copy api_key and api_token from below and paste in file name dev_(name) or env under services section api_key and api_token. \n")
                    print(f"api_key = {key}")
                    print(f"api_token = {secret}")
                    print("Api keys and token generated successfully.\n")
                else:
                    print("Api keys already exist.")
            else:
                print("Api key and api token is not set in application")
        except Exception as e:
            print(e)
