from libraries.system.model import BaseModel
from peewee import BooleanField, CharField, datetime
from playhouse.postgres_ext import BinaryJSONField, DateTimeTZField

class Users(BaseModel):
    class Meta:
        table_name = "users"

    email = CharField(unique=True, max_length=250, index=True)
    email_verified = BooleanField(default=False)
    name = CharField(max_length=250)
    password = CharField(max_length=250)
    status = CharField(choices=[('inactive', 'Pending Verification'), ('active', "Verified User"),
                                ('suspended', "Suspended User"), ('trash', "Trash")])
    suspend_reason = BinaryJSONField(null=True)
    authorization_token = CharField(max_length=250, null=True)
    password_reset_token = BinaryJSONField(null=True)
    agree_to_terms = BooleanField(default=False)
    activation_code = CharField(max_length=250, null=True)
    created_at = DateTimeTZField(null=True)
    updated_at = DateTimeTZField(default=datetime.datetime.now)
    deleted_at = DateTimeTZField(null=True)

class Migration(BaseModel):
    def up(self):
        """ Create user table. """
        Users.create_table()

    def down(self):
        Users.drop_table(cascade=True)