import sys, click, os, datetime
parent_dir = os.path.abspath(os.getcwd())
sys.path.append(parent_dir)
from libraries.system.application import Application
from my_package_profile_manager.libraries.email_templates.model import EmailTemplateModel

class DatabaseMigration(click.MultiCommand):

    def run_migration(self, action='up'):
        from my_package_profile_manager.migration.manager import ProfilLibMigrationManager
        migrationManager = ProfilLibMigrationManager()
        migrationManager.set_migration_action(action)
        migrationManager.run()


@click.group(chain=True)
def cli():
    pass

@cli.command('run-migration')
@click.option('--env', help='Application environment')
@click.option('--action', help='Methods defined in migration', default='up')
def migrate(env, action):
    application = Application()
    application.set_application_environment(env)
    application.init_settings()

    dbMigrationTool = DatabaseMigration()
    dbMigrationTool.run_migration(action)

@cli.command('profilelib-add-email-template')
@click.option('--env', help='Application environment')
def add_email_templates(env):
    application = Application()
    application.set_application_environment(env)
    application.init_settings()

    add_email_register_forget_templates()


def add_email_register_forget_templates():
    current_file = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    email_templates_directory=current_file+'/libraries/email_templates/email_template_files'

    for files in os.listdir(email_templates_directory):
        data: dict = {}
        with open(files, 'r') as f:
            name = f.replace(".txt", "")
            data['title'] = name
            data['subject']= name
            data['content'] = f.read()
            data['type']=name
            data['status']='active'
            data["created_at"] = datetime.datetime.now()
            data["updated_at"]= datetime.datetime.now()
            email_templates = EmailTemplateModel()
            email_templates.create()

if __name__ == '__main__':
    cli()